/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var Details = function()
{
	var details = {};
	
	
	details.initElements = function(element, autoInit)
	{
		if (autoInit == 'show-file')
		{
			Events.addEventHandler(element, 'click', details.showFileOnClick);
		}
		else if (autoInit == 'toggle-stacktrace')
		{
			Events.addEventHandler(element, 'click', details.toggleStackTraceOnClick);
		}
	}
	Doc.addElementInitializer(details.initElements);
	
	details.initPage = function()
	{
		Buttons.addClickHandler('btnInstall', details.installOnClick);
		Buttons.addClickHandler('btnScanResults', details.scanResultsOnClick);
		
		Buttons.addClickHandler('btnToggleExtension', details.toggleExtensionOnClick);
		Buttons.addClickHandler('btnToggleExtensionPoint', details.toggleExtensionPointOnClick);
		Buttons.addClickHandler('btnViewLog', details.viewLog);
		
		Buttons.addClickHandler('btnEnableFile', details.enableOrDisableFileOnClick);
		Buttons.addClickHandler('btnDisableFile', details.enableOrDisableFileOnClick);
	}

	
	details.installOnClick = function()
	{
		var url = 'index.jsp?ID='+App.getSessionId();
		url += '&cmd=ManualScan';
		Dialogs.openPopup(url, 'ManualScan', 1050, 700);
	}
	
	details.scanResultsOnClick = function()
	{
		var url = 'index.jsp?ID='+App.getSessionId();
		url += '&cmd=ScanResults';
		Dialogs.openPopup(url, 'ScanResults', 750, 500);
	}

	details.viewLog = function(event)
	{
		var target = event.currentTarget;
		var extensionId = Data.get(target, 'extension-id');
		var url = 'view_log.jsp?ID='+App.getSessionId();
		url += '&extensionId='+extensionId;
		Dialogs.openPopup(url, 'ViewLog'+extensionId, 1000, 600);
	}

	details.toggleExtensionOnClick = function(event)
	{
		var target = event.currentTarget;
		var extensionId = Data.get(target, 'extension-id');
		var isEnabled = Data.int(target, 'is-enabled');

		var iconName = isEnabled ? 'ExtensionDisabled' : 'Extension';
		window.parent.frames['tree'].Tree.setIcon(extensionId, iconName);

		var url = 'index.jsp?ID=' + App.getSessionId();
		url += '&cmd=EnableExtension&extensionId='+extensionId;
		url += '&enable='+(isEnabled ? 0 : 1);
		location.replace(url);
	}
	
	details.toggleExtensionPointOnClick = function(event)
	{
		var target = event.currentTarget;
		var extensionPointId = Data.get(target, 'extension-point-id');
		var isEnabled = Data.int(target, 'is-enabled');

		var iconName = isEnabled ? 'ExtensionPointDisabled' : 'ExtensionPoint';
		window.parent.frames['tree'].Tree.setIcon(extensionPointId, iconName);

		var url = 'index.jsp?ID=' + App.getSessionId();
		url += '&cmd=EnableExtensionPoint&extensionPointId='+extensionPointId;
		url += '&enable='+(isEnabled ? 0 : 1);
		location.replace(url);
	}

	details.enableOrDisableFileOnClick =  function(event)
	{
		var target = event.currentTarget;
		var fileName = Data.get(target, 'file-name');
		var fileUri = Data.get(target, 'file-uri');
		var enable = Data.int(target, 'enable');
		
		window.parent.frames['tree'].Tree.setChildIcons(fileName, enable);
		var url = 'index.jsp?ID='+App.getSessionId();
		url += '&cmd=EnableFile';
		url += '&fileuri='+encodeURIComponent(fileUri);
		url += '&enable='+enable;
		location.replace(url);
	}
	
	details.showFileOnClick = function(event)
	{
		var target = event.currentTarget;
		var fileUri = Data.get(target, 'file-uri');
		window.parent.frames['tree'].Tree.selectFile(fileUri);
	}
	
	details.toggleStackTraceOnClick = function(event)
	{
		var target = event.currentTarget;
		var stackTraceId = Data.get(target, 'stacktrace-id');
		Doc.showHide(stackTraceId);
	}
	
	return details;
}();

Doc.onLoad(Details.initPage);
