/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var Units = function()
{
	var units = {};
	
	/**
		Initialize the page.
	*/
	units.initPage = function()
	{
		var pageId = Doc.getPageId();		
		if (pageId == 'edit-page')
		{
			Buttons.addClickHandler('btnSave', units.save);
			Buttons.addClickHandler('close', App.closeWindow);
			
			// Tab validation
			TabControl.addTabValidator('settings.info', units.validateUnit);
			
			// Symbols
			Events.addEventHandler('symbols', 'change', units.symbolsOnChange);
		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var quantityId = Data.get('page-data', 'quantity-id');
			var attributes = {'item-type': 'UNIT', 'item-id': itemId, 'extra-url': '&quantity_id='+quantityId};
			
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnRestore', Buttons.restoreItem, attributes);
			Buttons.addClickHandler('btnUsingItems', Buttons.showUsingItems, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);
		}
	}

	units.validateUnit = function()
	{
		var frm = document.forms['unit'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		else if (Strings.trim(frm.symbols.value) == '')
		{
			Forms.showNotification(frm.symbols, 'You must enter at least one symbol');
			frm.symbols.focus();
			return false;
		}
		else if (isNaN(parseFloat(frm.referenceFactor.value)))
		{
			Forms.showNotification(frm.referenceFactor, "'" + frm.referenceFactor.value + "' is not a valid number");
			return false;
		}
		else if (isNaN(parseFloat(frm.referenceOffset.value)))
		{
			Forms.showNotification(frm.referenceOffset, "'" + frm.referenceOffset.value + "' is not a valid number");
			return false;
		}
		return true;
	}

	// Submit the form
	units.save = function()
	{
		var frm = document.forms['unit'];
		if (TabControl.validateActiveTab('settings'))
		{
			frm.submit();
		}
	}

	// Triggered when the list of symbols change
	// Picks the first one as the default symbol
	units.symbolsOnChange = function()
	{
		var frm = document.forms['unit'];
		var symbols = Strings.trim(frm.symbols.value).split('\n');
		var span = Doc.element('symbol');
		if (symbols[0])
		{
			span.innerHTML = symbols[0];
		}
	}

	return units;
}();
Doc.addElementInitializer(Forms.numberOnlyInitializer);
Doc.onLoad(Units.initPage);
