/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var Quota = function()
{
	var quota = {};
	
	/**
		Initialize the page.
	*/
	quota.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'edit-page')
		{
			Buttons.addClickHandler('btnSave', quota.save);
			Buttons.addClickHandler('close', App.closeWindow);
			
			// Tab validation
			TabControl.addTabValidator('settings.info', quota.validateQuota);
		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'QUOTA', 'item-id': itemId};
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnRestore', Buttons.restoreItem, attributes);
			Buttons.addClickHandler('btnUsingItems', Buttons.showUsingItems, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'QUOTA'};
			var tableAttributes = {'table-id': 'quota'};
			Buttons.addClickHandler('btnNewItem', Buttons.newItem, attributes);
			Buttons.addClickHandler('btnDeleteItems', Buttons.deleteItems, tableAttributes);
			Buttons.addClickHandler('btnRestoreItems', Buttons.restoreItems, tableAttributes);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);

			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
		}
	}
	
	quota.validateQuota = function()
	{
		var frm = document.forms['quota'];
		var totalQuotaId = Data.get('page-data', 'total-quota-id');
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name for the quota');
			return false;
		}
		if (Strings.trim(frm['PRIMARY:'+totalQuotaId].value) == '')
		{
			Forms.showNotification(frm['PRIMARY:'+totalQuotaId], 'You must enter a total primary quota');
			return false;
		}
		return true;
	}
	
	// Submit the form
	quota.save = function()
	{
		var frm = document.forms['quota'];
		if (TabControl.validateActiveTab('settings'))
		{
			frm.submit();
		}
	}
	
	// Handler for the 'unlimited' checkbox in the form
	quota.setUnlimited = function(event)
	{
		var frm = document.forms['quota'];
		var unlimited = event.currentTarget;
		var quotaInput = frm[unlimited.name.replace(/unlimited:/, '')];

		if (unlimited.checked)
		{
			Doc.addClass(quotaInput, 'disabled');
			quotaInput.disabled = true;
			quotaInput.oldValue = quotaInput.value;
			quotaInput.value = 'unlimited';
		}
		else
		{
			Doc.removeClass(quotaInput, 'disabled');
			quotaInput.disabled = false;
			quotaInput.value = quotaInput.oldValue ? quotaInput.oldValue : '';
		}
	}

	quota.initElements = function(element, autoInit)
	{
		if (autoInit != 'unlimited-quota') return;
		Buttons.addClickHandler(element, quota.setUnlimited);
	}
	
	return quota;
}();

Doc.addElementInitializer(Quota.initElements);
Doc.onLoad(Quota.initPage);
