<%-- $Id: list_tags.jsp 8153 2023-06-05 12:48:25Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Johan Enell, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.Tag"
	import="net.sf.basedb.core.ItemSubtype"
	import="net.sf.basedb.core.Annotation"
	import="net.sf.basedb.core.AnnotationType"
	import="net.sf.basedb.core.AnnotationSet"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.ItemResultIterator"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.Nameable"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.PluginDefinition"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.core.plugin.Plugin"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.snapshot.AnnotationLoaderUtil"
	import="net.sf.basedb.core.snapshot.AnnotationTypeFilter"
	import="net.sf.basedb.core.snapshot.AnnotationSnapshot"
	import="net.sf.basedb.core.snapshot.AnnotationSetSnapshot"
	import="net.sf.basedb.core.snapshot.SnapshotManager"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.PermissionUtil"
	import="net.sf.basedb.clients.web.ModeInfo"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Enumeration"
	import="net.sf.basedb.util.ShareableUtil"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.formatter.Formatter"
	import="net.sf.basedb.clients.web.formatter.FormatterFactory"
	import="net.sf.basedb.clients.web.extensions.ExtensionsControl"
	import="net.sf.basedb.clients.web.extensions.JspContext"
	import="net.sf.basedb.clients.web.extensions.renderer.PrefixSuffixRenderer"
	import="net.sf.basedb.clients.web.extensions.toolbar.ButtonAction" 
	import="net.sf.basedb.clients.web.extensions.toolbar.ToolbarUtil"
	import="net.sf.basedb.clients.web.extensions.list.ListColumnAction"
	import="net.sf.basedb.clients.web.extensions.list.ListColumnUtil"
	import="net.sf.basedb.clients.web.util.ProjectSpecificInfoFilter"
	import="net.sf.basedb.util.extensions.ExtensionsInvoker"
	import="java.util.Date"
	import="java.util.Iterator"
	import="java.util.List"
	import="java.util.Map"
	import="java.util.ArrayList"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="tbl" uri="/WEB-INF/table.tld" %>
<%@ taglib prefix="ext" uri="/WEB-INF/extensions.tld" %>
<%!
	private static final Item itemType = Item.TAG;
	private static final GuiContext guiContext = new GuiContext(itemType, GuiContext.Type.LIST);
%>
<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, Permission.DENIED, itemType);
final String ID = sc.getId();
final boolean createPermission = sc.hasPermission(Permission.CREATE, itemType);
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);

final ModeInfo mode = ModeInfo.get(request.getParameter("mode"));
final String callback = request.getParameter("callback");
final String title = mode.generateTitle("tag", "tags");
final DbControl dc = sc.newDbControl(":List "+itemType);
ItemResultIterator<Tag> tags = null;
List<AnnotationLoaderUtil> annotationLoaders = new ArrayList<AnnotationLoaderUtil>();
try
{
	Formatter<Date> dateFormatter = FormatterFactory.getDateFormatter(sc);

	final ItemQuery<ItemSubtype> typeQuery = ItemSubtype.getQuery(itemType);
	typeQuery.order(Orders.asc(Hql.property("name")));
	typeQuery.setCacheResult(true);
	final ItemQuery<AnnotationType> annotationTypeQuery = Base.getAnnotationTypesQuery(itemType);
	SnapshotManager manager = new SnapshotManager();
	ProjectSpecificInfoFilter psInfo = new ProjectSpecificInfoFilter();
	for (AnnotationType at : annotationTypeQuery.list(dc))
	{
		annotationLoaders.add(new AnnotationLoaderUtil(dc, manager, at));
	}

	Map<Plugin.MainType, Integer> pluginCount = PluginDefinition.countPlugins(dc, guiContext);
	JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, guiContext, null);
	try
	{
		final ItemQuery<Tag> query = Base.getConfiguredQuery(dc, cc, jspContext, true, Tag.getQuery(), mode);
		tags = query.iterate(dc);
	}
	catch (Throwable t)
	{
		cc.setMessage(t.getMessage());
	}
	int numListed = 0;
	ExtensionsInvoker<ButtonAction> invoker = ToolbarUtil.useExtensions(jspContext);
	ExtensionsInvoker<ListColumnAction<Tag,?>> columnsInvoker = ListColumnUtil.useExtensions(jspContext);
	%>
	<base:page title="<%=title==null ? "Tags" : title%>" type="<%=mode.getPageType()%>" id="list-page">
	<base:head scripts="table.js,~tags.js" styles="table.css,toolbar.css">
		<ext:scripts context="<%=jspContext%>" />
		<ext:stylesheets context="<%=jspContext%>" />
	</base:head>
	
	<base:body>
		<h1><%=title==null ? "Tags" : title%></h1>
		<div class="content">
		<tbl:table 
			id="taglist" 
			columns="<%=cc.getSetting("columns")%>"
			sortby="<%=cc.getSortProperty()%>" 
			direction="<%=cc.getSortDirection()%>"
			action="index.jsp"
			sc="<%=sc%>"
			item="<%=itemType%>"
			filterrows="<%=cc.getFilterRows()%>"
			subclass="fulltable"
			stickyheaders="<%=cc.getSetting("columns.sticky", "name")%>"
			>
			<tbl:hidden 
				name="mode" 
				value="<%=mode.getName()%>" 
			/>
			<tbl:hidden 
				name="callback" 
				value="<%=callback%>" 
				skip="<%=callback == null%>" 
			/>
			<tbl:columndef 
				id="name"
				property="name"
				datatype="string"
				title="Name"
				sortable="true" 
				filterable="true"
				exportable="true"
				show="always" 
			/>
			<tbl:columndef 
				id="id"
				clazz="uniquecol"
				property="id"
				datatype="int"
				title="ID"
				sortable="true"
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="owner"
				property="owner.name"
				datatype="string"
				title="Owner"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="entryDate"
				property="entryDate"
				datatype="date"
				title="Registered"
				sortable="true" 
				filterable="true"
				exportable="true"
				formatter="<%=dateFormatter%>"
			/>
			<%
			Enumeration<String, String> tagTypes = new Enumeration<String, String>();
			for (ItemSubtype subtype : typeQuery.list(dc))
			{
				tagTypes.add(Integer.toString(subtype.getId()), HTML.encodeTags(subtype.getName()));
			}
			%>		
			<tbl:columndef 
				id="itemSubtype"
				property="itemSubtype"
				sortproperty="itemSubtype.name"
				exportproperty="itemSubtype.name:string"
				datatype="int"
				enumeration="<%=tagTypes%>"
				title="Type"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="description"
				property="description"
				datatype="string"
				title="Description" 
				sortable="true" 
				filterable="true" 
				exportable="true"
			/>
			<%
			for (AnnotationLoaderUtil loader : annotationLoaders)
			{
				AnnotationType at = loader.getAnnotationType();
				Enumeration<String, String> annotationEnum = null;
				Formatter<Object> formatter = FormatterFactory.getTypeFormatter(sc, at.getValueType());
				if (at.isEnumeration())
				{
					annotationEnum = new Enumeration<String, String>();
					if (!at.getDisplayAsList()) annotationEnum.add("", "-none-");
					List<?> values = at.getValues();
					for (Object value : values)
					{
						String encoded = formatter.format(value);
						annotationEnum.add(encoded, encoded);
					}
				}
				%>
				<tbl:columndef 
					id="<%="at"+at.getId()%>"
					title="<%=HTML.encodeTags(at.getName())+" [A]"%>" 
					property="<%="#"+at.getId()%>"
					annotation="true"
					datatype="<%=at.getValueType().getStringValue()%>"
					enumeration="<%=annotationEnum%>"
					smartenum="<%=at.getDisplayAsList() %>"
					sortable="<%=at.getMultiplicity() == 1%>" 
					filterable="true" 
					exportable="true"
					formatter="<%=formatter%>"
					unit="<%=at.getDefaultUnit()%>"
				/>
				<%
			}
			%>	
			<tbl:columndef
				id="permission"
				title="Permission"
			/>
			<tbl:columndef
				id="sharedTo"
				title="Shared to"
				filterable="true"
				filterproperty="!sharedTo.name"
				datatype="string"
			/>
			<tbl:columndef 
				id="xt-columns" 
				extensions="<%=columnsInvoker%>" 
				jspcontext="<%=jspContext%>" 
			/>
			<div class="panelgroup bg-filled-50 bottomborder">
				<tbl:toolbar
					subclass="bottomborder"
					visible="<%=mode.hasToolbar()%>"
					>
					<tbl:button 
						id="btnNewItem"
						disabled="<%=!createPermission%>" 
						image="new.png" 
						title="New&hellip;" 
						tooltip="<%=createPermission ? "Create new tag" : "You do not have permission to create tags"%>" 
					/>
					<tbl:button 
						id="btnDeleteItems"
						image="delete.png"
						title="Delete" 
						tooltip="Delete the selected items" 
					/>
					<tbl:button 
						id="btnRestoreItems"
						image="restore.png"
						title="Restore" 
						tooltip="Restore the selected (deleted) items"
					/>
					<tbl:button 
						id="btnShareItems"
						image="share.png"
						title="Share&hellip;" 
						tooltip="Share the selected items"
					/>
					<tbl:button 
						id="btnSetOwner"
						image="take_ownership.png"
						title="Set owner&hellip;"
						tooltip="Change owner of the selected items"
					/>
					<tbl:button 
						id="btnColumns"
						image="columns.png" 
						title="Columns&hellip;" 
						tooltip="Show, hide and re-order columns" 
					/>
					<tbl:button 
						id="btnImport"
						data-plugin-type="IMPORT"
						image="import.png" 
						title="Import&hellip;" 
						tooltip="Import data" 
						visible="<%=pluginCount.containsKey(Plugin.MainType.IMPORT)%>"
					/>
					<tbl:button 
						id="btnExport"
						data-plugin-type="EXPORT"
						image="export.png" 
						title="Export&hellip;" 
						tooltip="Export data" 
						visible="<%=pluginCount.containsKey(Plugin.MainType.EXPORT)%>"
					/>
					<tbl:button 
						id="btnRunPlugin"
						data-plugin-type="OTHER"
						image="runplugin.png" 
						title="Run plugin&hellip;" 
						tooltip="Run a plugin" 
						visible="<%=pluginCount.containsKey(Plugin.MainType.OTHER)%>"
					/>
					<ext:render extensions="<%=invoker%>" context="<%=jspContext%>" 
						wrapper="<%=new PrefixSuffixRenderer<ButtonAction>(jspContext, "<td>", "</td>") %>"/>
				</tbl:toolbar>
				<tbl:panel>
					<tbl:presetselector />
					<tbl:navigator
						page="<%=cc.getPage()%>" 
						rowsperpage="<%=cc.getRowsPerPage()%>" 
						totalrows="<%=tags == null ? 0 : tags.getTotalCount()%>" 
						visible="<%=mode.hasNavigator()%>"
					/>
				</tbl:panel>
			</div>
			<tbl:data>
				<tbl:headers>
					<tbl:headerrow>
						<tbl:header clazz="row-index bg-filled-100" />
						<tbl:columnheaders />
					</tbl:headerrow>
					<%
					int numFilters = cc.getNumPropertyFilters();
					int numRows = cc.getFilterRows();
					for (int filterNo = 0; filterNo < numRows; filterNo++)
					{
						boolean lastRow = filterNo == numRows-1;
						%>
						<tbl:headerrow>
							<tbl:header subclass="row-index bg-filled-100">
								<div class="index-<%=mode.getName()%>">
									<div class="index"></div>
									<div class="check">
										<base:icon 
											subclass="link table-check"
											image="check_uncheck.png" 
											tooltip="Toggle all (use CTRL, ALT or SHIFT to check/uncheck)" 
											visible="<%=lastRow && mode.hasCheck()%>"
										/>
									</div>
									<div class="icons">
										<base:icon
											subclass="link table-filter-row-action"
											image="add.png"
											tooltip="Add extra filter row"
											visible="<%=lastRow%>"
										/><base:icon
											subclass="link table-filter-row-action"
											image="remove.png"
											tooltip="Remove this filter row"
											visible="<%=numRows > 1 || numFilters > 0 %>"
											data-remove-row="<%=filterNo%>"
										/>
									</div>
								</div>
							</tbl:header>
							<tbl:propertyfilter row="<%=filterNo%>" />
						</tbl:headerrow>
						<%
					}
					%>
					<tbl:columnsubtitles />
				</tbl:headers>
				<tbl:rows>
					<%
					if (cc.getMessage() != null)
					{
						%>
						<tbl:panel subclass="bg-filled-50">
							<div class="messagecontainer error"><%=cc.getMessage()%></div>
						</tbl:panel>
						<%
						cc.setMessage(null);
					}
					int index = cc.getPage()*cc.getRowsPerPage();
					int selectedItemId = cc.getId();
					if (tags != null)
					{						
						while (tags.hasNext())
						{
							Tag item = tags.next();
							int itemId = item.getId();	
							
							
							boolean deletePermission = item.hasPermission(Permission.DELETE);
							boolean sharePermission = item.hasPermission(Permission.SET_PERMISSION);	
							boolean usePermission = item.hasPermission(Permission.USE);
							boolean writePermission = item.hasPermission(Permission.WRITE);
							String tooltip = mode.isSelectionMode() ? 
									"Select this item" : "View this item" + (writePermission ? " (use CTRL, ALT or SHIFT to edit)" : "");
							String name = HTML.encodeTags(item.getName());
							index++;
							numListed++;
							%>
							<tbl:row>
								<tbl:header clazz="row-index bg-filled-100">
									<div class="index-<%=mode.getName()%>">
										<div class="index <%=index>999?"index-smaller":""%>"><%=index%></div>
										<div class="check">
											<base:input
												type="checkbox" 
												name="<%=itemId%>" 
												value="<%=itemId%>" 
												title="<%=name%>" 
												checked="<%=cc.getSelected().contains(itemId)%>"
												visible="<%=mode.hasCheck()%>"
											/>
											<base:input 
												type="radio" 
												name="item_id" 
												value="<%=itemId%>" 
												title="<%=name%>" 
												checked="<%=selectedItemId == itemId%>"
												visible="<%=mode.hasRadio()%>"
											/>
										</div>
										<div class="icons">
											<base:icon 
												image="deleted.png"
												id="<%="delete."+itemId %>"
												subclass="<%=deletePermission ? "table-delete-item" : "disabled" %>"
												data-item-id="<%=itemId%>"
												tooltip="This item has been scheduled for deletion" 
												visible="<%=item.isRemoved()%>"
											/><base:icon 
												image="shared.png" 
												id="<%="share."+itemId %>"
												subclass="<%=sharePermission ? "table-share-item" : "disabled" %>"
												data-item-id="<%=itemId%>"
												tooltip="This item is shared to other users, groups and/or projects" 
												visible="<%=item.isShared()%>"
											/>
										</div>
									</div>
								</tbl:header>
								<tbl:cell column="name"><div
									class="link table-item"
									data-item-id="<%=itemId%>"
									data-no-edit="<%=writePermission ? 0 : 1 %>" 
									tabindex="0"
									title="<%=tooltip%>"><%=name%></div></tbl:cell>
								<tbl:cell column="id"><%=item.getId()%></tbl:cell>
								<tbl:cell column="owner"
									><base:propertyvalue 
										item="<%=item%>" 
										property="owner"
										enableEditLink="<%=mode.hasEditLink()%>" 
										enablePropertyLink="<%=mode.hasPropertyLink()%>"
									/></tbl:cell>
								<tbl:cell column="entryDate" value="<%=item.getEntryDate()%>" />
								<tbl:cell column="itemSubtype"
									><base:propertyvalue 
										item="<%=item%>" 
										property="itemSubtype"
										enableEditLink="<%=mode.hasEditLink()%>" 
										enablePropertyLink="<%=mode.hasPropertyLink()%>"
									/></tbl:cell>
								<tbl:cell column="description"><%=HTML.encodeTags(item.getDescription())%></tbl:cell>
								<%
								if (item.isAnnotated())
								{
									AnnotationSetSnapshot snapshot = manager.getSnapshot(dc, item.getAnnotationSet().getId());
									for (AnnotationLoaderUtil loader : annotationLoaders)
									{
										%>
										<tbl:cell 
											column="<%="at"+loader.getId()%>"
											><%
											if (loader.find(snapshot, psInfo.reset())) 
											{
												%><tbl:cellvalue 
													list="<%=loader.getValues()%>"
													bulletlist="<%=loader.useBulletList() %>"
													suffix="<%=loader.getUnitSymbol()%>"
													clazz="<%=psInfo.hasProjectSpecificAnnotation() ? "ps-annotation" : null%>"
												/><%
											}
											%></tbl:cell>
										<%
									}
								}
								%>
								<tbl:cell column="permission"><%=PermissionUtil.getShortPermissions(item)%></tbl:cell>
								<tbl:cell column="sharedTo">
									<%
									Iterator<Nameable> sharees = ShareableUtil.getSharedTo(dc, item).iterator();
									while(sharees.hasNext())
									{
										Nameable n = sharees.next();
										if (mode.hasPropertyLink())
										{
											out.write(Base.getLinkedName(ID, n, false, mode.hasEditLink()));
										}
										else
										{
											out.write(HTML.encodeTags(n.getName()));
										}
										out.write(sharees.hasNext() ? ", " : "");
									}
									%>
								</tbl:cell>
								<tbl:xt-cells dc="<%=dc%>" item="<%=item%>">
									<tbl:cell column="xt-columns" />
								</tbl:xt-cells>
							</tbl:row>
							<%
							}
						}
					if (numListed == 0)
					{
						%>
						<tbl:panel subclass="bg-filled-50">
							<div class="messagecontainer note">
								<%=tags == null || tags.getTotalCount() == 0 ? "No tags were found" : "No tags on this page. Please select another page!" %>
							</div>
						</tbl:panel>
						<%
					}
					%>
				</tbl:rows>
			</tbl:data>
		</tbl:table>
		</div>
		<base:buttongroup subclass="dialogbuttons">
			<base:button id="btnOk" title="Ok" visible="<%=mode.hasOkButton()%>" />
			<base:button id="close" title="Cancel" visible="<%=mode.hasCancelButton()%>" />
			<base:button id="close" title="Close" visible="<%=mode.hasCloseButton()%>" />
		</base:buttongroup>
	</base:body>
	</base:page>
	<%
}
finally
{
	if (tags != null) tags.close();
	if (dc != null) dc.close();
}
%>
