/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2020 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var Configure = function()
{
	var configure = {};
	
	/**
		Initialize the page.
	*/
	configure.initPage = function()
	{
		Events.addEventHandler('presets', 'change', configure.presetSelected);
		Events.addEventHandler(document.body, 'click', configure.hideMessage);
		
		// Buttons
		Buttons.addClickHandler('close', App.closeWindow);
		Buttons.addClickHandler('btnAdd', configure.addColumn);
	}
	
	configure.presetSelected = function(event)
	{
		var frm = document.forms['linkedItems'];
		
		if (frm.presets.selectedIndex > 0)
		{
			var selected = frm.presets[frm.presets.selectedIndex];
			frm.linkName.value = Data.get(selected, 'linkname');
			Forms.selectListOption(frm.targetItemType, Data.get(selected, 'targettype'));
			frm.presets.selectedIndex = 0;
		}
	}
	
	configure.addColumn = function(event)
	{
		var frm = document.forms['linkedItems'];
		
		var data = {};
		data.linkName = frm.linkName.value;

		if (!data.linkName)
		{
			Forms.showNotification('linkName', 'A link name is requried!');
			return;
		}

		data.targetItemType = frm.targetItemType.value;
		data.property = '|'+data.linkName;
		if (data.targetItemType) 
		{
			data.property += '|'+data.targetItemType;
			// Store the selected target type in "recently used" list
			var url = 'ajax.jsp?ID='+App.getSessionId();
			url += '&cmd=AddRecentlyUsedLinkTargetType&itemType='+encodeURIComponent(data.targetItemType);
			var request = Ajax.getXmlHttpRequest();
			request.open("POST", url, true);
			request.send(null);
		}
		data.title = data.linkName;
		Events.sendCustomEvent(window.opener.Doc.element('selectLinkedItemColumn'), 'base-selected', data);
		
		frm.linkName.value = '';
		frm.linkName.focus();
		
		Doc.element('added-column-msg').innerHTML = '1 column has been added to the table.';
		Doc.show('added-column-msg');
		event.stopPropagation(); // To prevent the hideMessage() method from executing
	}
	
	configure.hideMessage = function(event)
	{
		Doc.hide('added-column-msg');
	}


	return configure;
}();

Doc.onLoad(Configure.initPage);